package com.sovelia.office;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import org.apache.poi.POIXMLProperties;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFFormulaEvaluator;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

public class XSSFFormulaTest {
	
	public static void main(String[] args) {
		String origInput = "originalFile.xlsx";
		String outOrigSuccess = "orig_success.xlsx";
		String outOrigFail = "orig_fail.xlsx";
		testSuccess(origInput, outOrigSuccess);
		/*will not fail*/
		testFail(origInput, outOrigFail);
		
		String modInput = "modified_in_MSOffice.xlsx";
		String outModSuccess = "mod_success.xlsx";
		String outModFail = "mod_fail.xlsx";
		testSuccess(modInput, outModSuccess);
		/*failed to evaluate the Array formula*/
		testFail(modInput, outModFail);
	}
	
	public static void testSuccess(String input, String output) {
		try {
			
			System.out.println("Starts success case..............");
			File file = new File(input);
			FileInputStream is = new FileInputStream(file);
			XSSFWorkbook workbook = (XSSFWorkbook) WorkbookFactory.create(is);
			evaluateAllValidFormulaCells(workbook);
			writeToFile(output, workbook);
			is.close();
			System.out.println("Ends success case..............");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void testFail(String input, String output) {
		try {
			
			System.out.println("Starts success case..............");
			File file = new File(input);
			FileInputStream is = new FileInputStream(file);
			XSSFWorkbook workbook = (XSSFWorkbook) WorkbookFactory.create(is);
			evaluateAllFormulaAtOnce(workbook);
			writeToFile(output, workbook);
			is.close();
			System.out.println("Ends success case..............");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void writeToFile(String filename, XSSFWorkbook workbook) throws Exception{
		FileOutputStream fos = new FileOutputStream(filename);
		workbook.write(fos);
		fos.close();
		workbook.close();
	}
	public static void writeCellValue(XSSFWorkbook workbook) {
		XSSFSheet sheet = workbook.getSheet( "Sheet1" );
		
		//CellReference cellReference = new CellReference("D4"); 
		Row row = sheet.getRow(2);
		Cell cell = row.getCell(4);
		if(cell==null) {
			cell = row.createCell(4,CellType.STRING);
		}
		cell.setCellValue("Test Value");
	}
	public static void writeCoreMeta(XSSFWorkbook workbook) {
		POIXMLProperties props = workbook.getProperties();
		/*Set core properties*/
        POIXMLProperties.CoreProperties coreProp = props.getCoreProperties();
        coreProp.setDescription("Apache POI 3.16/ Java 8");
	}
	
	public static void writeCustomMeta(XSSFWorkbook workbook) {
		POIXMLProperties props = workbook.getProperties();
        POIXMLProperties.CustomProperties custProp = props.getCustomProperties();
        /*set custom property*/
        custProp.addProperty("Typist", "me");
	}
	public static void evaluateAllFormulaAtOnce(XSSFWorkbook workbook) throws Exception{
		FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
		evaluator.evaluateAll();
	}
	
    private static void evaluateAllValidFormulaCells(XSSFWorkbook workbook) {
        XSSFFormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
        int numberOfSheets = workbook.getNumberOfSheets();
        for(int i=0; i<numberOfSheets; i++) {
            XSSFSheet sheet = workbook.getSheetAt(i);
            
            int lastRowNo=sheet.getLastRowNum();

            for(int rownum=0;rownum<=lastRowNo;rownum++){
                 XSSFRow row = sheet.getRow(rownum);
                 if (row!=null){
    
                      int lastCellNo=row.getLastCellNum();
    
                      for(int cellnum=0;cellnum<lastCellNo;cellnum++){
                          XSSFCell cell=row.getCell(cellnum);
                          if(cell!=null && cell.getCellTypeEnum()==CellType.FORMULA){
                             evaluateFormulaCell(cell, evaluator);
                          }
                       }
                 }
            }

        }
    }
    
    private static void evaluateFormulaCell(XSSFCell cell, XSSFFormulaEvaluator evaluator) {
        try {
        	System.out.println("**********************************'");
        	
            /*Broken cell value for Array*/
        	//evaluator.evaluateFormulaCell(cell);
    		//evaluator.evaluateFormulaCellEnum(cell);
        	
        	/*Correct cell value*/
            evaluator.evaluate(cell);
            System.out.println("Cell row:" + cell.getRowIndex() + " , col:" + cell.getColumnIndex());
            System.out.println("Cell Formula: " + cell.getCellFormula());
            switch(cell.getCachedFormulaResultTypeEnum()) {
            case NUMERIC:
                System.out.println("Last evaluated as: " + cell.getNumericCellValue());
                break;
            case STRING:
                System.out.println("Last evaluated as \"" + cell.getRichStringCellValue() + "\"");
                break;
                default :
                	System.out.println("Last evaluated formula as \"" + cell.getCachedFormulaResultTypeEnum() + "\"");
                	break;
    		}
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
